"use strict";
const RegClient = require('npm-registry-client');
const pauseStream = require('pause-stream');
const getAuthToken = require('registry-auth-token');
const logger = require('@zkochan/logger');
const defaults_1 = require('../defaults');
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = (opts) => {
    opts = opts || {};
    const client = new RegClient({
        retry: {
            count: opts.fetchRetries || defaults_1.default.fetchRetries,
            factor: opts.fetchRetryFactor || defaults_1.default.fetchRetryFactor,
            minTimeout: opts.fetchRetryMintimeout || defaults_1.default.fetchRetryMintimeout,
            maxTimeout: opts.fetchRetryMaxtimeout || defaults_1.default.fetchRetryMaxtimeout
        },
        log: Object.assign({}, logger, {
            verbose: logger.log.bind(null, 'verbose'),
            http: logger.log.bind(null, 'http')
        })
    });
    const cache = {};
    const get = (url, options) => {
        const key = JSON.stringify([url, options]);
        if (!cache[key]) {
            cache[key] = new Promise((resolve, reject) => {
                client.get(url, extend(url, options), (err, data, raw, res) => {
                    if (err)
                        return reject(err);
                    resolve(res);
                });
            });
        }
        return cache[key];
    };
    function getJSON(url, options) {
        const key = JSON.stringify([url, options]);
        if (!cache[key]) {
            cache[key] = new Promise((resolve, reject) => {
                client.get(url, extend(url, options), (err, data, raw, res) => {
                    if (err)
                        return reject(err);
                    resolve(data);
                });
            });
        }
        return cache[key];
    }
    const getStream = function (url, options) {
        return new Promise((resolve, reject) => {
            client.fetch(url, extend(url, options), (err, res) => {
                if (err)
                    return reject(err);
                const ps = pauseStream();
                res.pipe(ps.pause());
                resolve(ps);
            });
        });
    };
    /**
     * Extends request options with authorization headers
     */
    function extend(url, options) {
        options = options || {};
        const authToken = getAuthToken(url, { recursive: true });
        if (authToken) {
            options.headers = Object.assign({}, options.headers, {
                authorization: `${authToken.type} ${authToken.token}`
            });
        }
        return options;
    }
    return {
        get: get,
        getJSON: getJSON,
        getStream: getStream
    };
};
//# sourceMappingURL=got.js.map